@@ -9,13 +9,13 @@ if __name__ == "__main__":
 
     demo()
 
-import uuid
+# import uuid
 import re
+import json
+import csv
 
 from datetime import datetime
 
-from zzdb.utils import int_
-
 
 class Record:
     def __init__(self, t):
@@ -28,7 +28,6 @@ class Record:
 class ZzCursor:
     def __init__(self, zz_db, sql, table_name="", order="", where="", data=[], cache_flag=False):
         self.zz_db = zz_db
-        self.tempName = "_%s" % uuid.uuid4().hex.replace("-", "")
         self.sql = sql
         self.table_name = table_name
         self.order = order
@@ -36,6 +35,10 @@ class ZzCursor:
         self.where = where
         self.data = data
         self.cache_flag = cache_flag
+        # if self.zz_db.db_engine_name in ["mysql"] and self.cache_flag is True:
+        #     self.tempName = "_%s" % uuid.uuid4().hex.replace("-", "")
+        # else:
+        #     self.tempName = None
         self._rows = {}
         self._columns = []
         self._row_count = 0
@@ -116,9 +119,11 @@ class ZzCursor:
         _orAppend(rez)
         return " or ".join(_or)
 
-    def update(self, data):
+    def update(self, data, refresh=True, where=True):
         if self.table_name:
             rez = self.zz_db.update(self.table_name, data)
+            if refresh and rez:
+                self.refresh()
             return rez
 
     def insert(self, data, refresh=True, where=True):
@@ -132,13 +137,17 @@ class ZzCursor:
                 self.refresh()
             return rez
 
-    def delete(self, data):
+    def delete(self, data, refresh=True):
         if self.table_name:
             rez = self.zz_db.delete(self.table_name, data)
-            self.refresh()
+            if rez is True and refresh:
+                self.refresh()
             return rez
 
     def get(self, where="", column_name=""):
+        """returns value of given column or record dictionary
+        from first row  given table_name for where condition
+        """
         if self.table_name:
             return self.zz_db.get(self.table_name, where, column_name)
         return {}
@@ -180,11 +189,31 @@ class ZzCursor:
             if self.record(x)[pkName] == pkValue:
                 return x
 
+    def seek_row(self, data_dic):
+        row_counter = 0
+        for x in self.records():
+            if [z for z in data_dic if z in x and data_dic[z] == x[z]]:
+                return row_counter
+            row_counter += 1
+        return row_counter
+
     def get_primary_key_columns(self):
         return self.primary_key_columns[:]
 
     def refresh(self):
         self._currentRow = 0
+        if self.table_name:
+            self.primary_key_columns = [x for x in self.zz_db.get_primary_key_columns(self.table_name)]
+            self.sql = f"select * from {self.table_name}"
+            if self.where:
+                self.sql += f" where {self.where}"
+            if self.order:
+                self.sql += f" order by {self.order}"
+        self._rows = self.zz_db._cursor(f"""{self.sql}""", self.data)
+        if self.zz_db.last_sql_error or self._rows == {}:
+            self._row_count = -1
+        else:
+            self._row_count = len(self._rows)
         return self
 
     def row_count(self):
@@ -207,23 +236,83 @@ class ZzCursor:
     def get_columns(self):
         return [x for x in self.record(0)]
 
+    def _prepare_export(self, file):
+        rez = []
+        for x in self.records():
+            rez.append(x)
+        if not hasattr(file, "write"):
+            write_to = open(file, "w")
+        else:
+            write_to = file
+        return write_to, rez
+
+    def _prepare_import(self, file):
+        if not hasattr(file, "read"):
+            read_from = open(file)
+        else:
+            read_from = file
+        return read_from
+
+    def import_json(self, file):
+        """read json from file or file-like object
+
+        ;param file: str or file-like object
+        """
+        if self.table_name:
+            read_from = self._prepare_import(file)
+            rows = json.load(read_from)
+            for x in rows:
+                self.insert(x)
+
+    def export_json(self, file):
+        """write json into file or file-like object
+
+        ;param file: str or file-like object
+        """
+        write_to, rez = self._prepare_export(file)
+        if rez:
+            json.dump(rez, write_to, indent=1)
+
+    def export_csv(self, file):
+        """write csv(excel dialect) into file or file-like object
+
+        ;param file: str or file-like object
+        """
+        write_to, rez = self._prepare_export(file)
+        if rez:
+            csv_writer = csv.DictWriter(write_to, [x for x in rez[0]], dialect="excel")
+            csv_writer.writeheader()
+            for x in rez:
+                csv_writer.writerow(x)
+
+    def import_csv(self, file):
+        """read csv from file or file-like object
+
+        ;param file: str or file-like object
+        """
+        if self.table_name:
+            read_from = self._prepare_import(file)
+            rows = csv.DictReader(read_from, dialect="excel")
+            for x in rows:
+                self.insert(x)
+
 
 class ZzSqliteCursor(ZzCursor):
-    def refresh(self):
-        super().refresh()
-        if self.table_name:
-            self.primary_key_columns = [x for x in self.zz_db.get_primary_key_columns(self.table_name)]
-            self.sql = f"select * from {self.table_name}"
-            if self.where:
-                self.sql += f" where {self.where}"
-            if self.order:
-                self.sql += f" order by {self.order}"
-        self._rows = self.zz_db._cursor(f"""{self.sql}""", self.data)
-        if self.zz_db.last_sql_error or self._rows == {}:
-            self._row_count = -1
-        else:
-            self._row_count = len(self._rows)
-        return self
+    # def refresh(self):
+    #     super().refresh()
+    #     if self.table_name:
+    #         self.primary_key_columns = [x for x in self.zz_db.get_primary_key_columns(self.table_name)]
+    #         self.sql = f"select * from {self.table_name}"
+    #         if self.where:
+    #             self.sql += f" where {self.where}"
+    #         if self.order:
+    #             self.sql += f" order by {self.order}"
+    #     self._rows = self.zz_db._cursor(f"""{self.sql}""", self.data)
+    #     if self.zz_db.last_sql_error or self._rows == {}:
+    #         self._row_count = -1
+    #     else:
+    #         self._row_count = len(self._rows)
+    #     return self
 
     @staticmethod
     def get_table_names_sql(table_select_clause="", database_name=""):
@@ -246,120 +335,120 @@ class ZzSqliteCursor(ZzCursor):
 
 
 class ZzMysqlCursor(ZzCursor):
-    def __del__(self):
-        try:
-            self.removeTempTable()
-        except Exception:
-            pass
+    # def __del__(self):
+    #     try:
+    #         self.removeTempTable()
+    #     except Exception:
+    #         pass
 
-    def setCacheFlag(self, flag):
-        if flag != self.cacheFlag:
-            self._rows = {}
-        self.cacheFlag = flag
+    # def setCacheFlag(self, flag):
+    #     if flag != self.cacheFlag:
+    #         self._rows = {}
+    #     self.cacheFlag = flag
 
-    def record(self, rowNumber, columns={}):
-        if rowNumber >= 0 and rowNumber < self._row_count:
-            if rowNumber not in self._rows:
-                if self.primary_key_columns:
-                    row = self.zz_db._cursor(
-                        f"""select {self.table_name}.*
-                                            from {self.table_name},{self.tempName}
-                                            where {self.tempName}._rn_={rowNumber} and
-                                            {' and '.join([f'{self.table_name}.{x}={self.tempName}.{x}'
-                                                                for x in self.primary_key_columns])}
-                                        """
-                    )
-                else:
-                    row = self.zz_db._cursor(
-                        f"""select *
-                            from {self.tempName}
-                            where _rn_={rowNumber}"""
-                    )
-                if self.zz_db.last_sql_error:
-                    row = {0: {}}
-                if self.cache_flag:
-                    self._rows[rowNumber] = row[0]
-                # print (rowNumber,row)
-                rez = row[0]
-            else:
-                rez = self._rows[rowNumber]
-            # return rez
-            if columns:
-                return {x: rez[x] for x in rez if x in columns}
-            else:
-                return rez
+    # def record(self, rowNumber, columns={}):
+    #     if rowNumber >= 0 and rowNumber < self._row_count:
+    #         if rowNumber not in self._rows:
+    #             if self.primary_key_columns:
+    #                 row = self.zz_db._cursor(
+    #                     f"""select {self.table_name}.*
+    #                                         from {self.table_name},{self.tempName}
+    #                                         where {self.tempName}._rn_={rowNumber} and
+    #                                         {' and '.join([f'{self.table_name}.{x}={self.tempName}.{x}'
+    #                                                             for x in self.primary_key_columns])}
+    #                                     """
+    #                 )
+    #             else:
+    #                 row = self.zz_db._cursor(
+    #                     f"""select *
+    #                         from {self.tempName}
+    #                         where _rn_={rowNumber}"""
+    #                 )
+    #             if self.zz_db.last_sql_error:
+    #                 row = {0: {}}
+    #             if self.cache_flag:
+    #                 self._rows[rowNumber] = row[0]
+    #             # print (rowNumber,row)
+    #             rez = row[0]
+    #         else:
+    #             rez = self._rows[rowNumber]
+    #         # return rez
+    #         if columns:
+    #             return {x: rez[x] for x in rez if x in columns}
+    #         else:
+    #             return rez
 
-        else:
-            return {}
+    #     else:
+    #         return {}
 
-    def getPkRow(self, dataDic):
-        if self.primary_key_columns:
-            sql = f"""select _rn_
-                    from {self.tempName}
-                    where {' and '.join([f'{x}=%s' for x in self.primary_key_columns])}
-                        """
-            data = [dataDic[x] for x in self.primary_key_columns]
-            error, row = self.zz_db._cursor(sql, data)
-            if error:
-                return -1
-            else:
-                if row:
-                    return int_(row[0]["_rn_"])
-                else:
-                    return -1
-        else:
-            return -1
+    # def getPkRow(self, dataDic):
+    #     if self.primary_key_columns:
+    #         sql = f"""select _rn_
+    #                 from {self.tempName}
+    #                 where {' and '.join([f'{x}=%s' for x in self.primary_key_columns])}
+    #                     """
+    #         data = [dataDic[x] for x in self.primary_key_columns]
+    #         error, row = self.zz_db._cursor(sql, data)
+    #         if error:
+    #             return -1
+    #         else:
+    #             if row:
+    #                 return int_(row[0]["_rn_"])
+    #             else:
+    #                 return -1
+    #     else:
+    #         return -1
 
-    def removeTempTable(self):
-        if self.tempName and self.zz_db:
-            self.zz_db._cursor(f"drop temporary table IF EXISTS {self.tempName} ")
-        self._rows = {}
-        self._row_count = -1
+    # def removeTempTable(self):
+    #     if self.tempName and self.zz_db:
+    #         self.zz_db._cursor(f"drop temporary table IF EXISTS {self.tempName} ")
+    #     self._rows = {}
+    #     self._row_count = -1
 
-    def close(self):
-        self.removeTempTable()
-        self.zz_db = None
+    # def close(self):
+    #     self.removeTempTable()
+    #     self.zz_db = None
 
-    def refresh(self):
-        super().refresh()
-        if self.table_name:
-            self.primary_key_columns = [x for x in self.zz_db.get_primary_key_columns(self.table_name)]
-            if self.primary_key_columns:
-                self.sql = f"select {','.join(self.primary_key_columns)} from {self.table_name}"
-            else:
-                self.sql = f"select * from {self.table_name}"
-            if self.where:
-                self.sql += f" where {self.where}"
-            if self.order:
-                self.sql += f" order by {self.order}"
-        self.removeTempTable()
-        if self.sql.strip().upper().startswith("SELECT"):
-            self.zz_db._cursor(
-                f"""
-                            create temporary table {self.tempName}
-                            (
-                                select @i := @i + 1 AS _rn_, z1.*
-                                from  ({self.sql}) z1, (select @i:=-1) z2
-                            )
-                                """,
-                self.data,
-            )
-        else:
-            self.zz_db._cursor(f"""{self.sql}""", self.data)
-        if self.zz_db.last_sql_error:
-            self._row_count = -1
-        else:
-            self.zz_db._cursor(f"""alter table {self.tempName} add index (_rn_)""")
-            _row = self.zz_db._cursor(f"select count(1) as rc from {self.tempName}")
-            if self.zz_db.last_sql_error or _row == []:
-                self._row_count = -1
-            else:
-                self._row_count = int_(_row[0]["rc"])
-        return self
+    # def refresh(self):
+    #     super().refresh()
+    #     if self.table_name:
+    #         self.primary_key_columns = [x for x in self.zz_db.get_primary_key_columns(self.table_name)]
+    #         if self.primary_key_columns:
+    #             self.sql = f"select {','.join(self.primary_key_columns)} from {self.table_name}"
+    #         else:
+    #             self.sql = f"select * from {self.table_name}"
+    #         if self.where:
+    #             self.sql += f" where {self.where}"
+    #         if self.order:
+    #             self.sql += f" order by {self.order}"
+    #     self.removeTempTable()
+    #     if self.sql.strip().upper().startswith("SELECT"):
+    #         self.zz_db._cursor(
+    #             f"""
+    #                         create temporary table {self.tempName}
+    #                         (
+    #                             select @i := @i + 1 AS _rn_, z1.*
+    #                             from  ({self.sql}) z1, (select @i:=-1) z2
+    #                         )
+    #                             """,
+    #             self.data,
+    #         )
+    #     else:
+    #         self.zz_db._cursor(f"""{self.sql}""", self.data)
+    #     if self.zz_db.last_sql_error:
+    #         self._row_count = -1
+    #     else:
+    #         self.zz_db._cursor(f"""alter table {self.tempName} add index (_rn_)""")
+    #         _row = self.zz_db._cursor(f"select count(1) as rc from {self.tempName}")
+    #         if self.zz_db.last_sql_error or _row == []:
+    #             self._row_count = -1
+    #         else:
+    #             self._row_count = int_(_row[0]["rc"])
+    #     return self
 
     @staticmethod
     def get_table_names_sql(table_select_clause="", database_name=""):
-        return f"""select distinct table_name
+        return f"""select distinct table_name as table_name
                    FROM INFORMATION_SCHEMA.TABLES
                    WHERE table_schema='{database_name}' and
                        TABLE_TYPE<>'VIEW' {table_select_clause}
@@ -386,21 +475,21 @@ class ZzMysqlCursor(ZzCursor):
 
 
 class ZzPostgresqlCursor(ZzCursor):
-    def refresh(self):
-        super().refresh()
-        if self.table_name:
-            self.primary_key_columns = [x for x in self.zz_db.get_primary_key_columns(self.table_name)]
-            self.sql = f"select * from {self.table_name}"
-            if self.where:
-                self.sql += f" where {self.where}"
-            if self.order:
-                self.sql += f" order by {self.order}"
-        self._rows = self.zz_db._cursor(f"""{self.sql}""", self.data)
-        if self.zz_db.last_sql_error or self._rows == {}:
-            self._row_count = -1
-        else:
-            self._row_count = len(self._rows)
-        return self
+    # def refresh(self):
+    #     super().refresh()
+    #     if self.table_name:
+    #         self.primary_key_columns = [x for x in self.zz_db.get_primary_key_columns(self.table_name)]
+    #         self.sql = f"select * from {self.table_name}"
+    #         if self.where:
+    #             self.sql += f" where {self.where}"
+    #         if self.order:
+    #             self.sql += f" order by {self.order}"
+    #     self._rows = self.zz_db._cursor(f"""{self.sql}""", self.data)
+    #     if self.zz_db.last_sql_error or self._rows == {}:
+    #         self._row_count = -1
+    #     else:
+    #         self._row_count = len(self._rows)
+    #     return self
 
     @staticmethod
     def get_table_names_sql(table_select_clause="", database_name=""):
