@@ -3,9 +3,9 @@
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
 #    You may obtain a copy of the License at
-# 
+#
 #        http://www.apache.org/licenses/LICENSE-2.0
-# 
+#
 #    Unless required by applicable law or agreed to in writing, software
 #    distributed under the License is distributed on an "AS IS" BASIS,
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -21,44 +21,67 @@ if __name__ == "__main__":
 
     # from demo.demo_mysql import demo
     # from demo.demo_postgresql import demo
-    # from demo.demo_sqlite import demo
-    # from temp.try_pg_01 import demo
-    from tests.test_db import test_mysql, test_posrtgresql, test_sqlite
+    from demo.demo_sqlite import demo
 
-    test_mysql()
-    test_sqlite()
-    test_posrtgresql()
+    demo()
+    # from temp.try_pg_01 import demo
+
+    # from tests.test_db import test_mysql, test_postgresql, test_sqlite
+    # test_mysql()
+    # test_sqlite()
+    # test_postgresql()
 
 import re
-
 import sqlite3 as db_sqlite_connector
-import mysql.connector as db_mysql_connector
-import psycopg2 as db_postgresql_connector
 
+# import mysql.connector as db_mysql_connector
+# import psycopg2 as db_postgresql_connector
 
 from zzdb.utils import int_, is_sub_list
 from zzdb.cursor import ZzMysqlCursor, ZzSqliteCursor, ZzPostgresqlCursor
+from zzdb.schema import ZzDbSchema
 
 
 class ZzDb:
     def __init__(
         self,
-        db_engine_name,
+        db_engine_name=None,
         user=None,
         password=None,
         host=None,
         database_name=None,
         port=None,
         guest_mode=False,
+        url=None,
     ):
-        self.db_engine_name = db_engine_name
-        self.user = user
-        self.password = password
-        self.host = host
-        self.database_name = database_name
-        self.port = port
+        """
+        :param url: 'sqlite3|mysql|postgresql://username:password@host:port/database'
+            or
+        :param db_engine_name: 'sqlite3'|'mysql'|'postgresql', if None - 'sqlite3'
+        :param user=''
+        :param password=''
+        :param host=''
+        :param database_name='', if empty and db_engine_name == 'sqlite' - ':memory:'
+        :param port:''
+        :param guest_mode:, if empty - False
+        """
+        self.url = url
         self.guest_mode = guest_mode
-        self.connection = None
+        if url is not None:
+            self._parse_url()
+        else:
+            if db_engine_name is None:
+                db_engine_name = "sqlite3"
+                if database_name is None:
+                    database_name = ":memory:"
+            self.db_engine_name = db_engine_name
+            self.user = user
+            self.password = password
+            self.host = host
+            self.database_name = database_name
+            self.port = port
+
+            self.connection = None
 
         if self.db_engine_name not in ["mysql", "sqlite3", "postgresql"]:
             raise Exception(f"Sorry, wrong DBAPI engine - {self.db_engine_name}")
@@ -71,17 +94,30 @@ class ZzDb:
 
         self.db_schema = None
         self.connection = None
+        self.escape_char = '"'
 
         if self.db_engine_name == "mysql":
-            self.db_api_engine = db_mysql_connector
-            self.db_cursor_class = ZzMysqlCursor
+            try:
+                import mysql.connector as db_mysql_connector
+
+                self.db_api_engine = db_mysql_connector
+                self.db_cursor_class = ZzMysqlCursor
+                self.escape_char = "`"
+            except Exception:
+                raise Exception(
+                    "Sorry, can not import mysql.connector - use: pip install mysql-connector-python"
+                )
         elif self.db_engine_name == "postgresql":
-            self.db_api_engine = db_postgresql_connector
-            self.db_cursor_class = ZzPostgresqlCursor
+            try:
+                import psycopg2 as db_postgresql_connector
+
+                self.db_api_engine = db_postgresql_connector
+                self.db_cursor_class = ZzPostgresqlCursor
+            except Exception:
+                raise Exception("Sorry, can not import psycopg2 - use: pip install psycopg2-binary")
         elif self.db_engine_name == "sqlite3":
             self.db_api_engine = db_sqlite_connector
             self.db_cursor_class = ZzSqliteCursor
-
         try:
             self._connect()
             return
@@ -153,31 +189,56 @@ class ZzDb:
             connection = self.db_api_engine.connect(database=self.database_name, isolation_level=None)
         return connection
 
+    def _parse_url(self):
+        self.db_engine_name = self.url.split(":")[0]
+        self.user = self.url.split("//")[1].split(":")[0]
+        self.password = self.url.split("//")[1].split(":")[1].split("@")[0]
+        self.host = self.url.split("@")[1].split(":")[0]
+        self.port = self.url.split("@")[1].split(":")[1].split("/")[0]
+        self.database_name = self.url.split("@")[1].split("/")[1]
+
     def close(self):
         self.connection.close()
 
     def get_primary_key_columns(self, table_name=""):
         """returns database primary columns for given table"""
+        if self.db_schema is not None:
+            cols = self.db_schema.get_schema_table_attr(table_name)
+            rez = {}
+            for x in cols:
+                if cols[x].get("pk"):
+                    cols[x]["name"] = x
+                    rez[x] = cols[x]
+            return rez
+
         if self.db_engine_name in ("mysql", "postgresql"):
             return self.get_database_columns(table_name, "column_key='PRI'")
         elif self.db_engine_name == "sqlite3":
             return self.get_database_columns(table_name, "pk=1")
 
-    def get_database_columns(self, table_name="", filter=""):
+    def get_database_columns(self, table_name="", filter="", query_database=None):
         """returns database columns for given table"""
+
+        if self.db_schema is not None and filter == "" and query_database is None:
+            cols = self.db_schema.get_schema_table_attr(table_name)
+            for x in cols:
+                cols[x]["name"] = x
+            return cols
+
         cols = {}
         sql = self.db_cursor_class.get_table_columns_sql(table_name, filter, self.database_name)
+
         for x in self.cursor(sql).records():
             if "name" in x:
                 if "datalen" not in x:  # SQLITE
                     if "(" in x["datatype"]:
-                        ln = re.search(r"\((.*?)\)", x["datatype"]).group(1)
-                        if "," not in ln:
-                            x["datalen"] = ln
+                        field_length = re.search(r"\((.*?)\)", x["datatype"]).group(1)
+                        if "," not in field_length:
+                            x["datalen"] = field_length
                             x["datadec"] = "0"
                         else:
-                            x["datalen"] = ln.split(",")[0]
-                            x["datadec"] = ln.split(",")[1]
+                            x["datalen"] = field_length.split(",")[0]
+                            x["datadec"] = field_length.split(",")[1]
                         x["datatype"] = x["datatype"].split("(")[0]
                     else:
                         x["datalen"] = "0"
@@ -191,7 +252,7 @@ class ZzDb:
         sql = self.db_cursor_class.get_table_names_sql(table_select_clause, self.database_name)
         return [x["table_name"] for x in self.cursor(sql).records()]
 
-    def set_schema(self, db_schema):
+    def set_schema(self, db_schema: ZzDbSchema):
         """assign and migrate schema to database"""
         self.db_schema = db_schema
         return self.migrate_schema()
@@ -201,16 +262,14 @@ class ZzDb:
         if self.db_schema is None:
             return
         self.migrate_error_list = []
-        for table in self.db_schema.get_schema_table_attr():
+        for table in self.db_schema.get_schema_tables():
             # column that are already in
-            database_columns = self.get_database_columns(table)
-            # print(self.db_schema.get_schema_table_attr(table))
-            schema_columns = self.db_schema.get_schema_table_attr(table)
+            database_columns = self.get_database_columns(table, query_database=True)
+            schema_columns = self.db_schema.get_schema_columns(table)
             if not self.guest_mode:
                 self._add_zz_columns(schema_columns)
             for column in schema_columns:
-                # self.db_schema.refresh_foreignkey_column(table, column)
-                colDic = self.db_schema.get_schema_table_attr(table, column)
+                colDic = self.db_schema.get_schema_attr(table, column)
                 if column not in database_columns:  # new column
                     colDic["table"] = table
                     colDic["column"] = column
@@ -218,6 +277,7 @@ class ZzDb:
                 else:  # change schema as it is in database
                     colDic["datalen"] = database_columns[column]["datalen"]
                     colDic["datadec"] = database_columns[column]["datadec"]
+        self.migrate_indexes()
         return True
 
     def _add_zz_columns(self, schema_columns):
@@ -236,7 +296,7 @@ class ZzDb:
         column_definition["primarykey"] = "PRIMARY KEY" if column_definition.get("pk", "") else ""
         if column_definition.get("to_table") and column_definition.get("to_column"):
             # pull attributes from primary table
-            primary_column_definition = self.db_schema.get_schema_table_attr(
+            primary_column_definition = self.db_schema.get_schema_attr(
                 column_definition["to_table"], column_definition["to_column"]
             )
             for x in ["datatype", "datalen", "datadec"]:
@@ -266,20 +326,24 @@ class ZzDb:
         else:
             column_definition["size"] = ""
             column_definition["default"] = ""
-
-        sql_column_text = """{column} {datatype} {size} {primarykey} {default}""".format(**column_definition)
+        column_definition["escape_char"] = self.escape_char
+        sql_column_text = (
+            """ {escape_char}{column}{escape_char} {datatype} {size} {primarykey} {default}""".format(
+                **column_definition
+            )
+        )
         table = column_definition["table"]
         if table in self.get_tables():
-            sqlCmd = f"ALTER TABLE {table} ADD {sql_column_text}"
+            sql_cmd = f"ALTER TABLE {table} ADD {sql_column_text}"
         else:
-            sqlCmd = f"CREATE TABLE {table} ({sql_column_text})"
+            sql_cmd = f"CREATE TABLE {table} ({sql_column_text})"
 
-        self._cursor(sqlCmd)
-        if self.last_sql_error != "":
-            self.migrate_error_list.append(self.last_sql_error)
+        if not self.run_migrate_sql(sql_cmd):
             return False
-        # add the same colunm into log_ table
         if not self.guest_mode and not table.upper().startswith("LOG_"):
+
+            self.create_index(column_definition)
+
             log_column_definition = dict(column_definition)
             log_column_definition["pk"] = None
             log_column_definition["ai"] = None
@@ -288,6 +352,38 @@ class ZzDb:
             self.create_column(log_column_definition)
         return True
 
+    def create_index(self, column_definition):
+        """
+        create index for column
+        """
+        if (
+            column_definition.get("index")
+            or column_definition.get("column") == "name"
+            or column_definition.get("column") == "date"
+            or column_definition.get("to_table")
+        ):
+            sql_cmd = "CREATE INDEX {table}_{column} on {table} ({column})".format(**column_definition)
+            self.run_migrate_sql(sql_cmd)
+
+    def run_migrate_sql(self, sql_cmd):
+        self._cursor(sql_cmd)
+        # if 'Group' in sql_cmd:
+        #     print(sql_cmd)
+        if self.last_sql_error != "":
+            self.migrate_error_list.append(self.last_sql_error)
+            return False
+        return True
+
+    def migrate_indexes(self):
+        if self.guest_mode:
+            return
+        indexes = self.db_schema.get_schema_indexes()
+        for x in indexes:
+            if not x.get("name"):
+                x["name"] = re.sub(r"[^\w\d]+", "_", x["expression"])
+            sql_cmd = "CREATE INDEX {table}_{name} on {table} ({expression})".format(**x)
+            self.run_migrate_sql(sql_cmd)
+
     def _sqlite_patch(self, sql, record, table_columns):
         """Adapt sql statement for sqlite - convert str to int, replace placeholder character to ?"""
         for x in record:
@@ -304,12 +400,14 @@ class ZzDb:
         for x in record:
             if record[x] != "":
                 continue
-            datatype = self.db_schema.get_schema_table_attr(table, x).get("datatype")
+            datatype = self.db_schema.get_schema_attr(table, x).get("datatype")
+            if datatype is None:
+                continue
             if "int" in datatype or "dec" in datatype or "num" in datatype:
                 record[x] = "0"
 
     def insert(self, table_name="", record={}):
-        """insert dictionary to table"""
+        """insert dictionary into table"""
         if not (table_name and record):
             return None
 
@@ -319,20 +417,24 @@ class ZzDb:
             # check foreign keys
             foreign_keys_list = self.db_schema.get_primary_tables(table_name, record)
             for x in foreign_keys_list:
+                x["escape_char"] = self.escape_char
                 if (
-                    self.get(x["primary_table"], "%(primary_column)s='%(child_value)s'" % x, x["child_value"])
+                    self.get(
+                        x["primary_table"],
+                        "{escape_char}{primary_column}{escape_char}= '{child_value}' ".format(**x),
+                        x["child_value"],
+                    )
                     == {}
                 ):
                     self.last_sql_error = (
                         "Foreign key error for insert:"
-                        + f" For {table_name}"
-                        + ".{child_column}".format(**x)
+                        + f" For {self.escape_char}{table_name}{self.escape_char}"
+                        + ".{escape_char}{child_column}{escape_char}".format(**x)
                         + " not found value '{child_value}' ".format(**x)
                         + "in table "
                         + x["primary_table"]
                         + ".{primary_column}".format(**x)
                     )
-                    # print(self.last_sql_error)
                     self.last_error_data = x
                     return False
 
@@ -343,16 +445,29 @@ class ZzDb:
         for x in primary_key_columns:
             if x not in columns_list:
                 columns_list.append(x)
-            primary_key_value = int_(record.get(x, 0))
+            is_string_data = True if ("char" in primary_key_columns[x]["datatype"]) else False
+            if is_string_data:
+                primary_key_value = record.get(x, "")
+            else:
+                primary_key_value = int_(record.get(x, 0))
             while (
-                self.cursor(sql=f"select {x} from {table_name} where {x}={primary_key_value}").row_count() > 0
+                self.cursor(
+                    sql=f"""select {self.escape_char}{x}{self.escape_char}
+                            from {table_name} 
+                            where {self.escape_char}{x}{self.escape_char}='{primary_key_value}'
+                        """
+                ).row_count()
+                > 0
             ):
-                primary_key_value += 1
+                if is_string_data:
+                    primary_key_value += "_"
+                else:
+                    primary_key_value += 1
             record[x] = primary_key_value
 
         sql = (
             f"insert into {table_name} ("
-            + ",".join([x for x in columns_list])
+            + ",".join([f"{self.escape_char}{x}{self.escape_char}" for x in columns_list])
             + ") values ("
             + ",".join(["%s" for x in columns_list])
             + ")"
@@ -404,9 +519,15 @@ class ZzDb:
         columns_list = [x for x in record if x in table_columns]
         if is_sub_list(primary_key_columns.keys(), record.keys()):
             sql = f"update {table_name} set " + ",".join(
-                [" %s=%%s " % x for x in record if x not in primary_key_columns and x in columns_list]
+                [
+                    f" {self.escape_char}{x}{self.escape_char}=%s "
+                    for x in record
+                    if x not in primary_key_columns and x in columns_list
+                ]
+            )
+            sql += " where " + " and ".join(
+                [f" {self.escape_char}{x}{self.escape_char} = %s " for x in primary_key_columns]
             )
-            sql += " where " + " and ".join([f"{x}=%s " for x in primary_key_columns])
             if self.db_engine_name == "sqlite3":
                 sql = self._sqlite_patch(sql, record, table_columns)
 
@@ -422,31 +543,35 @@ class ZzDb:
                 if not table_name.upper().startswith("LOG_") and table_name.upper() != "PLATFORM":
                     self.insert("log_" + table_name, record)
                 return True
+        else:
+            self.last_sql_error = "Update requires a primary key column!"
 
     def delete(self, table_name="", record={}):
         if not (table_name and record):
             return None
         self.last_error_data = {}
         for x in self.db_schema.get_child_tables(table_name, record):
+            x["escape_char"] = self.escape_char
             rez = self._cursor(
-                "select 1 from {child_table} where {child_column}='{parent_value}'".format(**x)
+                "select 1 from {child_table} where {escape_char}{child_column}{escape_char}='{parent_value}'".format(
+                    **x
+                )
             )
             if {} != rez:
-
                 self.last_sql_error = (
                     "Foreign key error for delete:"
-                    + f" Value in {table_name}"
-                    + ".{parent_column}".format(**x)
+                    + f" Row in {self.escape_char}{table_name}{self.escape_char}"
+                    + ".{escape_char}{parent_column}{escape_char}".format(**x)
                     + "={parent_value}".format(**x)
-                    + " can not to be deleted, beacause "
-                    + " it used in table {child_table}.{child_column}".format(**x)
+                    + " can not to be deleted, because "
+                    + ' it used in table "{child_table}"."{child_column}"'.format(**x)
                 )
                 self.last_error_data = x
 
                 return False
         table_columns = self.get_database_columns(table_name)
         columns_list = [x for x in record if x in table_columns]
-        where_clause = " and ".join([f"{x}=%s " for x in columns_list])
+        where_clause = " and ".join([f"{self.escape_char}{x}{self.escape_char} = %s " for x in columns_list])
         data = [record[x] for x in columns_list]
 
         select_sql = f"select * from {table_name} where {where_clause}"
@@ -471,8 +596,7 @@ class ZzDb:
             return True
 
     def get(self, table_name="", where="", column_name=""):
-        """
-        return value of given column or record dictionary
+        """returns value of given column or record dictionary
         from first row  given table_name for where condition
         """
         column_name = f"({column_name}) as ret " if column_name else "*"
@@ -527,3 +651,13 @@ class ZzDb:
             data=data,
             cache_flag=cache_flag,
         )
+
+    def table(self, table_name="", order="", where="", cache_flag=True):
+        return self.db_cursor_class(
+            self,
+            sql="",
+            table_name=table_name,
+            order=order,
+            where=where,
+            cache_flag=cache_flag,
+        )
